//========================================================================================
//   NOM     : commandThread.cpp  		                                                                    
//   Date    : 16/04/2003                                                                     
//   Version : 1.01                                                                       
//   Auteur  : Sebastien JEFFROY                                                                          
//   Email   : sj29fr@yahoo.fr
//   Site    : http://sjeffroy.free.fr
//
//--------- GNU GPL LICENCE --------------------------------------------------
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
//----------------------------------------------------------------------------
//
//   Description : Routines de commande ihm lcd + infrarouge + menus
//----------------------------------------------------------------------------
#include "commandThread.h"

// images 122x32
#include "lcd_images.cpp"


//===========================================================================
// Thread pour lire les parametres de Winamp toutes les 1 seconde
//===========================================================================
// Constructeur
CommandThread::CommandThread(QDialog *dialog) 
{
	_track = -1;
	_pwm = 255;		// backlight max
	_dialog = dialog;
}

CommandThread::~CommandThread() {
	stop();
}

//---------------------------------------------------------------------------
void 
CommandThread::setLblTitre(QLabel *lblTitre, QLabel *lblInfo, QTextView* TxtEeprom)
{
	_lblTitre = lblTitre;
	_lblInfo = lblInfo;
	_txtEeprom = TxtEeprom;
}
		
//---------------------------------------------------------------------------
void 
CommandThread::setRefresh(unsigned int msec) 
{
	_msec = msec;
}

//---------------------------------------------------------------------------
void 
CommandThread::setPortCom(SerialCOM * portCom)
{
	_portCom = portCom;
	_lcd = new Lcd(_portCom);


}

//---------------------------------------------------------------------------
// Code Infrarouge
unsigned long 
CommandThread::TelecoIR(int *touche, int menu) {
		QString str;
		unsigned long irCode=0, irCodeSave;
		int repeatIR = 0;
		_lcd->readIRretry(&irCode);

		*touche = 0xFFFF;

		repeatIR = 0;

		MediaPlayer mplayer;
		Ktv ktv;

		switch(irCode) {
			// Telecommande
			//-------------

			// Menus touche
			//-------------
			case 0x02100000 :				// TUNER				
				*touche = TEL_TUNER;
				break;
			case 0x03300000:				// DSR/AUX
				//*touche = TEL_AUX;
				if (_pwm>16) _pwm = 16;
				else _pwm = 255;
				_lcd->PWMretry(_pwm);
				break;
			case 0x03900000:				// Touche CD 
				*touche = TEL_CD;
				break;
			case 0x02700000:				// TAPE
				*touche = TEL_TAPE;
				break;
			case 0x03000000:				// DCC
				*touche = TEL_DCC;
				break;
			case 0x03C00000:				// PHONO
				*touche = TEL_PHONO;
				break;


			// Commandes permamnentes
			//-----------------------
			// KTV chaines +/-
			//----------------
			case 0x02120400 :				// ST-				
				ktv.cmdPrevious();
				if (menu==TEL_TUNER) _lcd->TextRetry("CH-  ", font8x8p, 3, 30);
				break;
			case 0x02120000 :				// ST+				
				ktv.cmdNext();
				if (menu==TEL_TUNER) _lcd->TextRetry("CH+  ", font8x8p, 3, 30);
				break;

			// Backlight +/-
			//--------------
			case 0x03380400:				// ST2 -
				if (_pwm-16>0) _pwm -= 16;
				else _pwm = 0;
				_lcd->PWMretry(_pwm);
				repeatIR = 1;
				break;
			case 0x03380000:				// ST2 +
				if (_pwm+16<=255) _pwm += 16;
				else _pwm = 255;
				_lcd->PWMretry(_pwm);
				repeatIR = 1;
				break;

			// MediaPlayer (nouvelle version) Ajouter POWERDVD ???
			//----------------------------------------------------
			case 0x03930200 :				// Stop CD
				mplayer.cmdStop();
				if (menu==TEL_CD) _lcd->TextRetry("Stop ", font8x8p, 3, 30);
				break;
			case 0x03930C00 :				// Play CD
				mplayer.cmdPlay();
				if (menu==TEL_CD) _lcd->TextRetry("Play ", font8x8p, 3, 30);
				break;
			case 0x03980400 :				// Previous CD
				mplayer.cmdPrevious();
				if (menu==TEL_CD) _lcd->TextRetry("Prev.", font8x8p, 3, 30);
				break;
			case 0x03920000:				// Pause CD
				mplayer.cmdFullScreen();// ???? ou => display mode
				if (menu==TEL_CD) _lcd->TextRetry("Pause", font8x8p, 3, 30);
				break;
			case 0x03980000 :				// Next CD
				mplayer.cmdNext();
				if (menu==TEL_CD) _lcd->TextRetry("Next ", font8x8p, 3, 30);
				break;

			// Winamp
			//-------
			case 0x027C0000:				// Previous TAPE
				_winCMD.cmdPrevious();	// previous Track
				break;
			case 0x02730C00:				// Next TAPE
				_winCMD.cmdNext();		// next Track
				break;
			case 0x02730200:				// Stop TAPE
				_winCMD.cmdStop();		// Stop
				break;
			case 0x02720000:				// Pause TAPE
				_winCMD.cmdPlay();		// Play/Pause
				break;
			case 0x02730800 :				// avance 5s (forward TAPE)
				_winCMD.cmdForward();
				repeatIR = 1;
				break;
			case 0x02720600 :				// recule 5s (rewind TAPE)
				_winCMD.cmdRewind();
				repeatIR = 1;
				break;
			case 0x02730000:				// Rec TAPE
				_winCMD.cmdShuffle();	// Shuffle (random pistes)
				break;


			// Commandes permanente modifiees suivant menu courant
			//----------------------------------------------------
			case 0x02018000 :				// vol +
				switch(menu) {
					// KTV
					case TEL_TUNER	:
						ktv.cmdIncVolume();
						_lcd->TextRetry("Vol+ ", font8x8p, 3, 30);
						break;

					// backlight : pas de changement menu!!
					case TEL_AUX :
						break;
					
					// MediaPlayer
					case TEL_CD :
						mplayer.cmdIncVolume();
						_lcd->TextRetry("Vol+ ", font8x8p, 3, 30);
						break;

					// Winamp
					case TEL_TAPE :
					// RFU 
					case TEL_DCC :
					// RFU
					case TEL_PHONO :
						_winCMD.cmdIncVolume();
						break;
				}
				repeatIR = 1;
				break;
			case 0x02018400 :				// vol -
				switch(menu) {
					// KTV
					case TEL_TUNER	:								
						ktv.cmdDecVolume();
						_lcd->TextRetry("Vol- ", font8x8p, 3, 30);
						break;

					// backlight : pas de changement menu!!
					case TEL_AUX :
						break;
					
					// MediaPlayer
					case TEL_CD :
						mplayer.cmdDecVolume();
						_lcd->TextRetry("Vol- ", font8x8p, 3, 30);
						break;

					// Winamp
					case TEL_TAPE :
					// RFU 
					case TEL_DCC :
					// RFU
					case TEL_PHONO :
						_winCMD.cmdDecVolume();
						break;
				}
				repeatIR = 1;
				break;
			case 0x02004C00 :				// Mute
				switch(menu) {
					// KTV
					case TEL_TUNER	:	
						ktv.cmdMuteVolume();
						_lcd->TextRetry("Mute ", font8x8p, 3, 30);
						break;

					// backlight : pas de changement menu!!
					case TEL_AUX :
						break;
					
					// MediaPlayer
					case TEL_CD :
						mplayer.cmdMuteVolume();
						_lcd->TextRetry("Mute ", font8x8p, 3, 30);
					break;

					// Winamp
					case TEL_TAPE :
					// RFU 
					case TEL_DCC :
					// RFU
					case TEL_PHONO :
						_winCMD.cmdMuteVolume();
						break;
				}
				break;
			case 0x02004000:				// DISPLAY MODE => full screen
				switch(menu) {
					// KTV
					case TEL_TUNER	:	
						ktv.cmdFullScreen();
						_lcd->TextRetry("Full.", font8x8p, 3, 30);
						break;

					// backlight : pas de changement menu!!
					case TEL_AUX :
						break;
					
					// MediaPlayer
					case TEL_CD :
						mplayer.cmdFullScreen();
						_lcd->TextRetry("Full.", font8x8p, 3, 30);
						break;

					// Winamp
					case TEL_TAPE :
						_winCMD.cmdFullScreen();
					// RFU 
					case TEL_DCC :
					// RFU
					case TEL_PHONO :
						break;
				}
				break;
			
				
				break;

			//###############

			case 0x02000100 :				// '1'				
				*touche = 1;
				break;
			case 0x02000600 :				// '2'				
				*touche = 2;
				break;
			case 0x02000400 :				// '3'				
				*touche = 3;
				break;
			case 0x02001800 :				// '4'
				*touche = 4;
				break;
			case 0x02001C00 :				// '5'
				*touche = 5;
				break;
			case 0x02001200 :				// '6'
				*touche = 6;
				break;
			case 0x02001000 :				// '7'
				*touche = 7;
				break;
			case 0x02006000 :				// '8'
				*touche = 8;
				break;
			case 0x02006400 :				// '9'
				*touche = 9;
				break;
			case 0x02000000 :				// '0'				
				*touche = 0;
				break;

			case 0x02040C00 :				// Veille O-				
				if (_dialog->isHidden()) _dialog->show();
				else _dialog->hide();
				break;

			// Boutons poussoir		
			case 0x88888800 :				// BP_UP
				_winCMD.cmdNext();		// next Track
				break;
			case 0x88888801 :				// BP_DOWN	
				_winCMD.cmdPrevious();	// previous Track
				break;
			case 0x88888803 :				// BP_PLAY 								
				_winCMD.cmdPlay();		// Play/Pause
				break;
			case 0x88888804 :				// BP_EJECT 
				if (_dialog->isHidden()) _dialog->show();
				else _dialog->hide();
				break;	
			
			default :
				repeatIR = 1;				// sinon blocage affichage
				break;
		}

		// Sauvegarde code ir
		irCodeSave = irCode;

		// Antirebond touche
		int cptNoIR = 0;
		if ((irCode!=0xFFFFFFFF)&&(repeatIR==0)) {
			if ((irCode & 0x88888800) == 0x88888800) _lcd->BeepRetry(0); 
			do {
				_lcd->readIRretry(&irCode, 15);
				if (irCode==0xFFFFFFFF) {
					cptNoIR++;
					Sleep(10);					// tempo 10ms pour reception codeIR suivant
				}
				else cptNoIR = 0;
			}while(cptNoIR<11);//10 //50);
		}

	return irCodeSave;
}


//---------------------------------------------------------------------------
// Update Afficheur LCD :
// Menu Winamp
void 
CommandThread::WinampMenu(void) {
	int flagTrack = 0;

	// Modifier strName 40 caracteres pour effacer texte precedant
	bool statTitle = true;
	
	// Numero piste, Play/Pause/Stop
	int trkNb, Status;
	char buffer[1024];
	_winCMD.getTrackData(&trkNb, &Status, buffer);		


	// Nom piste
	QString strName;
	_winCMD.getTitle(strName);

	if (_track!=trkNb) {
		flagTrack = 1;
		char lcdTab[41];
		for(int idx = 0; idx<41; idx++) {
			if (strName.at(idx) != 0) {
				lcdTab[idx] = strName.at(idx).latin1();
			}
			else lcdTab[idx] = ' ';
		}
		lcdTab[40] = '\0';
		_lcd->TextRetry(lcdTab, font5x7p, 0, 0, 10);		// 10 retry
	}

	// Temps chanson
	char hh = 0, mm, ss, mss, mmLen, ssLen;
	_winCMD.getSongPosAsNum(&mm, &ss, &mss);		// temps courant chanson
	_winCMD.getSongLenAsNum(&mmLen, &ssLen);		// temps total chanson
	
	QString tps;
	
	if (Status == WINAMP_STATUS_STOP || mm>99 || mm<0 || ss>=60 || ss<0) {mm=0; ss=0;}
	if (mmLen>99 || ssLen >=60 || mmLen<0 || ssLen<0) {mmLen=0; ssLen = 1;}
	if (_track!=trkNb) {		// "mm:ss/mm:ss"
		tps.sprintf("%2.1d:%2.2d/%2.1d:%2.2d", mm, ss, mmLen, ssLen);
		statTitle = _lcd->TextRetry((char*)(tps.latin1()), font5x7p, 2, 0);
	}
	else {						// "mm:ss"
		tps.sprintf("%2.1d:%2.2d", mm, ss);
		statTitle = _lcd->TextRetry((char*)(tps.latin1()), font5x7p, 2, 0);
	}
	if (mmLen==0 && ssLen==0) _lcd->BarTimeRetry(0, 2, 11*6+4);
	else _lcd->BarTimeRetry((mm*60+ss)*255/(mmLen*60+ssLen), 2, 11*6+4);

	int h, m;
	QTime time;					// heure du systeme
	time = QTime::currentTime();
	h =  time.hour();
	m = time.minute();
	tps.sprintf("%2.1d:%2.2d", h, m);
	_lcd->TextRetry((char*)(tps.latin1()), font5x7n, 2, 122-5*6);


	// Volume
	_lcd->BarRetry(_winCMD.getMute() ? 0 : _winCMD.getVolume(), 3, 121-22);

	// Debit, Frq Ech, ST/Mo
	QString info;
	if (_track!=trkNb) {		// "128KBPS 44KHz ST"
		info.sprintf("%3.2dKBPS %2.2dKHz %s", _winCMD.getBitRate(), 
			_winCMD.getSampleRate(), _winCMD.getChannels()==2 ? "ST" : "Mo");
		_lcd->TextRetry((char*)(info.latin1()), font5x7p, 3, 0);
	}
	else {						// "128"
		info.sprintf("%3.2d", _winCMD.getBitRate() );
		_lcd->TextRetry((char*)(info.latin1()), font5x7p, 3, 0);
	}
	
	if ((statTitle==true)&&(flagTrack)) {
		flagTrack = 0;
		_track = trkNb;
	}
}

//---------------------------------------------------------------------------
// Menu Heure Systeme
void 
CommandThread::DateMenu(void) {

	QString str;
	
	// heure du systeme
	int h, m, s;
	QTime time;					
	time = QTime::currentTime();
	h =  time.hour();
	m = time.minute();
	s = time.second();
	str.sprintf(" %2.1d:%2.2d.%2.2d  ", h, m, s);
	_lcd->TextRetry((char*)(str.latin1()), font8x8n, 0, 0);

	// date systeme
	QDate date;								
	date = QDate::currentDate();

	str.sprintf("%2.1d/%2.2d/%d ", date.day(), date.month(), date.year() );
	_lcd->TextRetry((char*)(str.latin1()), font8x8n, 1, 0);

	// Centre cercle
	unsigned char cx = 15, cy = 121-15;
	// Rectangle a effacer
	//_lcd->RectFillRetry(0,121-15-15, 31,121, 0);
	
	double angle, h2;//, pi = 3.141592897932384626433832795;
	// Cercle
	_lcd->CercleRetry(cx,cy,15);	// cx, cy,radius

	// Effacer aiguilles precedantes si necessaire
	if (h!=hLast) {
		h2 = hLast%12 + mLast/60.0;
		angle = M_PI - (h2*2*M_PI/12.0);
		_lcd->LigneRetry(cx,cy, cx+8*cos(angle), cy+ 8*sin(angle), 0);	// x1,y1, x2,y2, fill
		hLast = h;	
	}
	
	if (m!=mLast){
		angle = M_PI - mLast*2*M_PI/60.0;
		_lcd->LigneRetry(cx,cy, cx+11*cos(angle), cy+ 11*sin(angle), 0);	// x1,y1, x2,y2, fill
		mLast = m;
	}
	
	if (s!=sLast) {
		angle = M_PI - sLast*2*M_PI/60.0;
		_lcd->LigneRetry(cx,cy, cx+14*cos(angle), cy+ 14*sin(angle), 0);	// x1,y1, x2,y2, fill
		sLast = s;
	}


	h2 = h%12 + m/60.0;
	angle = M_PI - (h2*2*M_PI/12.0);

	// Heure
	_lcd->LigneRetry(cx,cy, cx+8*cos(angle), cy+ 8*sin(angle), 1);	// x1,y1, x2,y2, fill

	// Minutes
	angle = M_PI - m*2*M_PI/60.0;
	_lcd->LigneRetry(cx,cy, cx+11*cos(angle), cy+ 11*sin(angle), 1);	// x1,y1, x2,y2, fill

	// Secondes
	angle = M_PI - s*2*M_PI/60.0;
	_lcd->LigneRetry(cx,cy, cx+14*cos(angle), cy+ 14*sin(angle), 1);	// x1,y1, x2,y2, fill

	// temperature
	signed char temp;
	_lcd->readTempRetry(&temp, 0);		// capteur adr 0
	str.sprintf(" %+2.2dC ", temp);

	_lcd->TextRetry((char*)(str.latin1()), font8x8n, 3, 0);
	
}

//---------------------------------------------------------------------------
// Menu Temperature (+ heure)
void 
CommandThread::TemperatureMenu(void) {

	QString str;
	
	// heure du systeme
	int h, m, s;
	QTime time;					
	time = QTime::currentTime();
	h =  time.hour();
	m = time.minute();
	s = time.second();

	if (s!=sLast) {

		str.sprintf(" %2.1d:%2.2d.%2.2d ", h, m, s);		// " XX:XX.XX "
		_lcd->TextRetry((char*)(str.latin1()), font8x8n, 0, 121-10*8);

		// temperature
		signed char temp;
		_lcd->readTempRetry(&temp, 0);		// capteur adr 0
		str.sprintf(" %+2.2d ", temp);		// - C

		_lcd->TextRetry((char*)(str.latin1()), font8x8n, 2, 60);
		

		sLast = s;
	}
}

//---------------------------------------------------------------------------
// Update IHM PC
void 
CommandThread::WinampMenuIHMPC(unsigned long codeir) {

	// Titre Piste
	QString strName;
	_winCMD.getTitle(strName);
	_lblTitre->setText(strName);

	// Debit, Frq, Nb Canaux
	int sample = _winCMD.getSampleRate();	// 44   (KHz)
	int bRate = _winCMD.getBitRate();		// 128  (KBPS)
	int nbCh = _winCMD.getChannels();		// 2

	// Temps piste
	char songPos[64], songLen[64];	
	_winCMD.getSongPosAsText(songPos);		// Temps courant de la chanson
	_winCMD.getSongLenAsText(songLen);		// Temps total de la chanson

	// Numero piste, Play/Pause/Stop
	int trkNb, Status;
	char buffer[1024];
	_winCMD.getTrackData(&trkNb, &Status, buffer);

	QString str;
	str.sprintf ( "%d KBPS  %d KHz  %d CH  Pos=%s  Len=%s  TrkNb=%d  Status=%d ir=%8.8X", 
						bRate,   sample, nbCh,  songPos, songLen, trkNb, Status, codeir);
	_lblInfo->setText(str);

}

//---------------------------------------------------------------------------
////#define DEBUG
void 
CommandThread::run()
{
	unsigned long irCodeLast=0xFFFFFFFF;
	int touche = 0xFFFF, menu = TEL_TAPE/*1*/, menuLast = 0xFFFF, flagBmp = 1; 

	_stopThread = 0;
	if (_msec==0) _msec = 200;					// 200ms

	#ifdef DEBUG 	
	_FstdErr = fopen("D:\\stderr.txt", "w+");	// fichier debug
	#endif

	MediaPlayer mplayer;
	Ktv ktv;


	while(!_stopThread) 
	{
		msleep(_msec);								// 200ms
		
		_winCMD.getWinamp();						// recupere Winamp 
		mplayer.getMediaPlayer();
		ktv.getKtv();

		// Code Infrarouge
		unsigned long irCode;

		irCode = TelecoIR(&touche, menu);
		// Sauvegarde pour affichage code touche
		if (irCode!=0xFFFFFFFF) irCodeLast = irCode;

		// Update IHM PC
		WinampMenuIHMPC(irCodeLast);

		// Menu en fonction de touche
		// Update Afficheur LCD
		if (menu!=menuLast) {
			_lcd->ClearRetry();
			menuLast = menu;
			_track = -1;
			_lcd->BeepRetry(0);
			_lcd->PWMretry(_pwm);
			flagBmp = 1;
		}
		
		switch(menu) {
			/*case 0 : 
				break;
			case 1 : 
				break;
			case 2 : 
				break;
			case 3 :
				break;
			case 4 : 
				break;*/

			// KTV
			case TEL_TUNER	:								
				if (flagBmp == 1) {
					// bitmap PC => lcd
					_lcd->Bmp122x32Retry(image_ktv);
					flagBmp = 0;
				}
				if (ktv.KtvFound()==0)
					_lcd->TextRetry("Err. ", font8x8p, 3, 30);
				break;

			// backlight : pas de changement menu!!
			case TEL_AUX :
				/*if (flagBmp == 1) {
					// bitmap
					_lcd->BitmapEeRetry(44);			// Ampoule ON
					flagBmp = 0;
				}
				break;*/
			
			// MediaPlayer
			case TEL_CD :
				if (flagBmp == 1) {
					// bitmap PC => lcd
					_lcd->Bmp122x32Retry(image_mplayer);
					flagBmp = 0;
				}
				if (mplayer.MediaPlayerFound()==0) 
					_lcd->TextRetry("Err. ", font8x8p, 3, 30);
				break;

			// Winamp
			case TEL_TAPE :
				WinampMenu();
				if (_winCMD.winampFound()==0)
					//               123456789012345 
					_lcd->TextRetry("Winamp Erreur  ", font8x8p, 0, 0);
				break;

			// RFU 
			case TEL_DCC :
				DateMenu();
				break;
			
			// RFU
			case TEL_PHONO :
				if (flagBmp == 1) {
					// bitmap
					_lcd->BitmapEeRetry(39);			// thermometre
					flagBmp = 0;
				}
				TemperatureMenu();
				break;

			default :
				menu = TEL_TAPE;
				break;
		}
		if ((touche>=0 && touche <=9) || 
			 (touche>=TEL_TUNER && touche<=TEL_PHONO && touche!=TEL_AUX)) 
					menu = touche;

		
	}
		
  	#ifdef DEBUG 	
	fclose(_FstdErr);
	#endif
}


//---------------------------------------------------------------------------
// Stoppe le thread
//---------------------------------------------------------------------------
void CommandThread::stop() 
{
	_stopThread = 1;
}


