#include "lcd.h"



//----------------------------------------------------------------------------
// Constructeur recupere port com
//----------------------------------------------------------------------------
Lcd::Lcd(SerialCOM * portCom) {
	_portCom = portCom;
	ClearRetry();
}

//----------------------------------------------------------------------------
// destructeur
//----------------------------------------------------------------------------
Lcd::~Lcd() {


}

//----------------------------------------------------------------------------
// Envoie d'un commande (avec calcul checksum + verif checksum recu)
//----------------------------------------------------------------------------
bool
Lcd::send(char *data, unsigned char len) {
	char chkSum, vChkSum, chkData[10], iSum;
	int tmpHexa;

	chkSum = 0;
	for(int i=0; i<len; i++) {
		chkSum += data[i];
	}
	data[i] = chkSum;
	
	//testIR();									// test reception code IR

	_portCom->com_purge();					// vide tampon
	_portCom->com_putstring(data, i+1);	// envoie

	// Reception Checksum
	for(iSum=0; iSum<3; iSum++) {
		chkData[iSum] = _portCom->com_getchar_live();
		if (_portCom->serial_get_error()!=0) {
			_portCom->serial_clear_error();
			return false;
		}
	}

	sscanf(chkData, "%x", &tmpHexa);   
	vChkSum = tmpHexa & 0xFF;
	
	// Combien ????
	//Sleep(10);// 10ms //
	//_portCom->com_msleep(5);

	if (chkSum==vChkSum) return true;
	return false;
}

//----------------------------------------------------------------------------
// Lecture temperature
//----------------------------------------------------------------------------
bool
Lcd::readTempRetry(signed char *temp, unsigned char adr, int nbRetry) {
	bool status;
	do {
		status = readTemp(temp, adr);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}

bool 
Lcd::readTemp(signed char *temp, unsigned char adr) {
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 4+1;							// longueur 
	data[2] = RS_TEMP;					// fonction
	data[3] = adr&0x07;					// adresse capteur

	// calcul du checksum + envoie avec verif checksum
	if (send(data, 4)==false) return false;
		
	// Reception Temperature
	*temp = _portCom->com_getchar_live();
	if (_portCom->serial_get_error()!=0) {
		*temp = 0;
		return false;
	}
	return true;
}


//----------------------------------------------------------------------------
// Test si reception d'un code IR (ircode 4 octets) 
//----------------------------------------------------------------------------
bool 
Lcd::readIRretry(unsigned long *ircode, int nbRetry) {
	bool status;
	do {
		status = readIR(ircode);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}

bool
Lcd::readIR(unsigned long *ircode) {
	//char codeIR[9];
	char data[96];
	unsigned long ir;

	data[0] = 0x01;						// start
	data[1] = 3+1;							// longueur 
	data[2] = RS_CODE_IR;				// fonction

	*ircode = 0xFFFFFFFF;
	// calcul du checksum + envoie avec verif checksum
	if (send(data, 3)==false) return false;
		
	// Reception Infrarouge
	//codeIR[0]
	ir = _portCom->com_getchar_live()  * 0x1000000;
	if (_portCom->serial_get_error()!=0) return false;
	//codeIR[1]
	ir += _portCom->com_getchar_live() * 0x10000;
	if (_portCom->serial_get_error()!=0) return false;
	//codeIR[2]
	ir += _portCom->com_getchar_live() * 0x100;
	if (_portCom->serial_get_error()!=0) return false;
	//codeIR[3]
	ir += _portCom->com_getchar_live();
	if (_portCom->serial_get_error()!=0) return false;
	//codeIR[4] = '\0';
	*ircode = ir;
	return true;
}

//----------------------------------------------------------------------------
// Affiche le texte text
//----------------------------------------------------------------------------
bool
Lcd::TextRetry(char * text, unsigned char font, unsigned char ligne,
					unsigned char col, int nbRetry) {
	bool status;
	do {
		status = Text(text, font, ligne, col);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}


bool
Lcd::Text(char * text, unsigned char font, 
				unsigned char ligne, unsigned char col) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = strlen(text) + 7;		// longueur (6 info + checksum)
	data[2] = RS_LCD_TEXT;				// fonction
	data[3] = font;						// police
	data[4] = ligne;						// ligne
	data[5] = col;							// colonne
	strcpy(&data[6], text);				// copie du texte

	// calcul du checksum + envoie avec verif checksum
	return send(data, strlen(text)+6);

}

//----------------------------------------------------------------------------
// Efface le LCD
//----------------------------------------------------------------------------
bool
Lcd::ClearRetry(int nbRetry) {
	bool status;
	do {
		status = Clear();
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}


bool
Lcd::Clear(void) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 0x04;						// longueur
	data[2] = RS_LCD_CLEAR;				// fonction
	// calcul du checksum + envoie avec verif checksum
	return send(data, 3);

}


//----------------------------------------------------------------------------
// Affiche le bitmap (61x8)
//----------------------------------------------------------------------------
bool
Lcd::BmpRetry(char * bitmap, unsigned char zone, int nbRetry) {
	bool status;
	do {
		status = Bmp(bitmap, zone);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}


bool
Lcd::Bmp(char * bitmap, unsigned char zone) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 61 + 5;						// longueur (4 info + checksum)
	data[2] = RS_LCD_GRAPH;				// fonction
	data[3] = zone;						// zone graph (0-7)
	memcpy(&data[4], bitmap, 61);		// copie du bmp

	// calcul du checksum + envoie avec verif checksum
	return send(data, 61+4);

}

//----------------------------------------------------------------------------
// Affiche le bitmap (122x32)
//----------------------------------------------------------------------------
bool
Lcd::Bmp122x32Retry(const unsigned char *image, int nbRetry) {
	bool status;
	do {
		status = Bmp122x32(image);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}

int
Lcd::Bmp122x32(const unsigned char *image) {

// affiche un bitmap 61 pixels * 8 sur le LCD 
// X |--------|--------|
// 0 |   0    |    1   |
//   |--------|--------|
// 1 |   2    |    3   |
//   |--------|--------|
// 2 |   4    |    5   |
//   |--------|--------|
// 3 |   6    |    7   |
//   |--------|--------|
	int stat = 0, idx=0;
	for (int i=0; i<8; i++) {
		stat += Bmp((char*)(&image[61*i]), idx);
		idx+=2;
		if (idx==8) idx = 1;
	}
	return stat;
	
}


//----------------------------------------------------------------------------
// Affiche point
//----------------------------------------------------------------------------
bool
Lcd::PlotRetry(unsigned char x1, unsigned char y1, unsigned char fill, int nbRetry) {
	bool status;
	do {
		status = Plot(x1,y1,fill);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::Plot(unsigned char x1, unsigned char y1, unsigned char fill) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 5 +1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_PLOT;				// fonction
	data[3] = x1;							
	data[4] = y1;							
	//data[5] = fill;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 5);

}

//----------------------------------------------------------------------------
// Affiche ligne
//----------------------------------------------------------------------------
bool
Lcd::LigneRetry(unsigned char x1, unsigned char y1,
					unsigned char x2, unsigned char y2, unsigned char fill, int nbRetry) {
	bool status;
	do {
		status = Ligne(x1,y1,x2,y2, fill);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::Ligne(unsigned char x1, unsigned char y1,
					unsigned char x2, unsigned char y2, unsigned char fill) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 8 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_LIGNE;				// fonction
	data[3] = x1;							
	data[4] = y1;							
	data[5] = x2;							
	data[6] = y2;							
	data[7] = fill;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 8);

}


//----------------------------------------------------------------------------
// Affiche rectangle
//----------------------------------------------------------------------------
bool
Lcd::RectRetry(unsigned char x1, unsigned char y1,
					unsigned char x2, unsigned char y2, int nbRetry) {
	bool status;
	do {
		status = Rect(x1,y1,x2,y2);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::Rect(unsigned char x1, unsigned char y1,
					unsigned char x2, unsigned char y2) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 7 +1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_RECT;				// fonction
	data[3] = x1;							
	data[4] = y1;							
	data[5] = x2;							
	data[6] = y2;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 7);

}

//----------------------------------------------------------------------------
// Affiche rectanglePlein (0:efface, 1:rempli)
//----------------------------------------------------------------------------
bool 
Lcd::RectFillRetry(unsigned char x1, unsigned char y1,
						 unsigned char x2, unsigned char y2, unsigned char fill, int nbRetry) {
	bool status;
	do {
		status = RectFill(x1,y1,x2,y2,fill);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;

}

bool 
Lcd::RectFill(unsigned char x1, unsigned char y1,
				  unsigned char x2, unsigned char y2, unsigned char fill) {

	char data[96];

	data[0] = 0x01;						// start
	data[1] = 8 +1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_RECT_FILL;		// fonction
	data[3] = x1;							
	data[4] = y1;							
	data[5] = x2;							
	data[6] = y2;							
	data[7] = fill;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 8);
}

//----------------------------------------------------------------------------
// Affiche cercle
//----------------------------------------------------------------------------
bool
Lcd::CercleRetry(unsigned char cx, unsigned char cy,
					unsigned char radius, int nbRetry) {
	bool status;
	do {
		status = Cercle(cx,cy,radius);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::Cercle(unsigned char cx, unsigned char cy, unsigned char radius) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 6 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_CERCLE;				// fonction
	data[3] = cx;							
	data[4] = cy;							
	data[5] = radius;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 6);

}


//----------------------------------------------------------------------------
// Affiche Bitmap EEPROM
//----------------------------------------------------------------------------
bool
Lcd::BitmapEeRetry(unsigned char num,int nbRetry) {
	bool status;
	do {
		status = BitmapEe(num);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::BitmapEe(unsigned char num) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 4 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_BITMAP;			// fonction
	data[3] = num;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 4);

}



//----------------------------------------------------------------------------
// Beep
//----------------------------------------------------------------------------
bool
Lcd::BeepRetry(unsigned char num,int nbRetry) {
	bool status;
	do {
		status = Beep();
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::Beep() {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 3 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_BEEP;			// fonction

	// calcul du checksum + envoie avec verif checksum
	return send(data, 3);

}

//----------------------------------------------------------------------------
// PWM backlight
//----------------------------------------------------------------------------
bool 
Lcd::PWMretry(unsigned char pwm,int nbRetry) {
	bool status;
	do {
		status = PWM(pwm);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}


bool 
Lcd::PWM(unsigned char pwm) {
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 4 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_PWM;						// fonction
	data[3] = pwm;				

	// calcul du checksum + envoie avec verif checksum
	return send(data, 4);
}

//----------------------------------------------------------------------------
// Affiche barregraph valeur, ligne, col
//----------------------------------------------------------------------------
bool
Lcd::BarRetry(unsigned char valeur, unsigned char ligne,
					unsigned char col, int nbRetry) {
	bool status;
	do {
		status = Bar(valeur,ligne,col);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::Bar(unsigned char valeur, unsigned char ligne, unsigned char col) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 6 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_BAR;				// fonction
	data[3] = valeur;							
	data[4] = ligne;							
	data[5] = col;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 6);

}

//----------------------------------------------------------------------------
// Affiche barregraph temps valeur, ligne, col
//----------------------------------------------------------------------------
bool
Lcd::BarTimeRetry(unsigned char valeur, unsigned char ligne,
					unsigned char col, int nbRetry) {
	bool status;
	do {
		status = BarTime(valeur,ligne,col);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
}



bool
Lcd::BarTime(unsigned char valeur, unsigned char ligne, unsigned char col) {
	
	char data[96];

	data[0] = 0x01;						// start
	data[1] = 6 + 1 ;						// longueur (dont checksum) 
	data[2] = RS_LCD_BAR_TIME;			// fonction
	data[3] = valeur;							
	data[4] = ligne;							
	data[5] = col;							

	// calcul du checksum + envoie avec verif checksum
	return send(data, 6);

}





//----------------------------------------------------------------------------
// Ecrit 32 octets max en hexa (ascii) 
//----------------------------------------------------------------------------
bool
Lcd::WriteEEretry(int adr, char *data, int nbData, int nbRetry) {
	bool status;
	do {
		status = WriteEE(adr, data, nbData);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
	return true;
}

bool
Lcd::WriteEE(int adr, char *data, int nbData) {
/*	char sum = 0;
	char dataC[1024];	// buffer tamporaire	avant transfert
	dataC[0] = 'W';
	dataC[1] = hexa2ascii((adr >> 12) & 0x0F);	// adrH MSB 
	dataC[2] = hexa2ascii((adr >> 8) & 0x0F);		// adrH LSB
	dataC[3] = hexa2ascii((adr >> 4) & 0x0F);		// adrL MSB
	dataC[4] = hexa2ascii(adr & 0x0F);				// adrL LSB
	int cpt=5;
	for(int i=0; i<nbData; i++) {
		dataC[cpt] = hexa2ascii((data[i]>>4) & 0x0F);
		cpt++;
		dataC[cpt] = hexa2ascii(data[i]&0x0F);
		sum += data[i];
		cpt++;
	}
	dataC[cpt++] = '\n';

	//_portCom->com_in_clear();						// vide tampon RX
	_portCom->com_purge();

	_portCom->com_putstring(dataC, cpt);			// transfert dataC

	char chkSum, nbDat;
	int iSum=0, tmpHexa;
	char chkData[50];

	for(iSum=0; iSum<9; iSum++) {
		chkData[iSum] = _portCom->com_getchar_live();
		if (_portCom->serial_get_error()!=0) return false;
	}

	sscanf(chkData, "%x", &tmpHexa);   
	//nbDat = ascii2hexa(chkData[2]) * 16 + ascii2hexa(chkData[3]);
	//chkSum = ascii2hexa(chkData[4]) * 16 + ascii2hexa(chkData[5]);
	nbDat = tmpHexa/256L;
	chkSum = tmpHexa & 0xFF;
	
	Sleep(5);// ms //_portCom->com_msleep(5);

	if ((nbDat==nbData*2) && (sum==chkSum)) return true;
	return false;*/
	return true;
}
		
//----------------------------------------------------------------------------
// Lit 32 octets en hexa (ascii) 
//----------------------------------------------------------------------------
bool
Lcd::ReadEEretry(int adr, char *data, int nbRetry) {
	bool status;
	do {
		status = ReadEE(adr, data);
		nbRetry--;
		if (status==false) {
			//_portCom->com_msleep(5);		// tempo 5ms si pb
			Sleep(3);
		}
	} while ((nbRetry!=0) && (!status));
	return status;
	return true;
}


bool
Lcd::ReadEE(int adr, char *data) {
	// 32 octets
	char dataC[96];

	dataC[0] = (char)'R';
	dataC[1] = hexa2ascii((adr >> 12) & 0x0F);	// adrH MSB 
	dataC[2] = hexa2ascii((adr >> 8) & 0x0F);		// adrH LSB
	dataC[3] = hexa2ascii((adr >> 4) & 0x0F);		// adrL MSB
	dataC[4] = hexa2ascii(adr & 0x0F);				// adrL LSB



	// Reception donnees
	for(int i=0; i<32; i++) {
		char tmpH, tmpL;
		tmpH = _portCom->com_getchar_live();
		if (_portCom->serial_get_error()!=0) return false;
		tmpL = _portCom->com_getchar_live();
		if (_portCom->serial_get_error()!=0) return false;
		data[i] = ascii2hexa(tmpH)*16 + ascii2hexa(tmpL);
	}

	return false;
}

