#ifndef LCD_H
#define LCD_H

#include <limits.h>

///#include <qmainwindow.h>
/*#include <qthread.h>


// ----Prototypes of Inp and Outp---		// LINK inpout32.lib + inpout32.dll
short _stdcall Inp32(short PortAddress);
void _stdcall Out32(short PortAddress, short data);

#define LPT_BASE 0x378				// I/O
#define LPT_STATUS (LPT_BASE+1)	// INPUT
#define LPT_CTRL (LPT_BASE+2)		// OUTPUT

// --- STATUS
#define LPT_ST_ERROR		0x08
#define LPT_ST_SELECT	0x10
#define LPT_ST_PAPER_OUT 0x20
#define LPT_ST_ACK		0x40
#define LPT_ST_BUSY		0x80		// Inverse

// --- CTRL
#define LPT_CT_STROBE	0x01
#define LPT_CT_AUTOFEED	0x02		// Inverse
#define LPT_CT_INIT		0x04
#define LPT_CT_SEL_IN	0x08		// Inverse


#define LCD_WRITE LPT_CT_STROBE
#define LCD_READ LPT_CT_AUTOFEED
#define LCD_CD LPT_CT_INIT	// 1 command 0 data
#define LCD_BUSY LPT_ST_BUSY

void
lcd_putdata(char data);

void
lcd_putcmd(char data);

void
lcd_goto(unsigned char ligne, unsigned char col);

void
lcd_puts(const char * str);

void
lcd_string_loc_lc(const char *s, unsigned char ligne, unsigned char col);


void
lcd_init(void);*/


//========================================================================================
// PSP LCD
//========================================================================================
enum {
	LCD_MODE_TEXT = 1,
	LCD_MODE_GRAPH = 2,
	LCD_MODE_EQU = 3,		// equalizer
	LCD_CLEAR = 4,
	LCD_GOTO = 5,
	LCD_SET_MP3 = 6,
	LCD_PLAY_PAUSE = 7
	
};

//========================================================================================
// LCD RS232
//========================================================================================
enum {
	RS_READ_MP3_TITLE = 't',					 
	RS_LCD_GRAPH = 'P',
	RS_MODE_EQU = 'E',			// equalizer
	RS_LCD_CLEAR = 'C',
	RS_GOTO = 'G',
	RS_SET_MP3 = 'M',
	RS_READI2C_ASCII = 'R',		// 24c256
	RS_READI2C_HEXA = 'r',
	RS_WRITEI2C_ASCII = 'W',
	RS_WRITEI2C_HEXA = 'w',
	RS_READEE_PIC_ASCII = 'L',	// 'L' lit EEPROM interne PIC 
	RS_READEE_PIC_HEXA = 'l',
	RS_LCD_TEXT = 'T',
	RS_LCD_LIGNE = 'z',
	RS_LCD_RECT = 'q',
	RS_LCD_RECT_FILL = 'Q',
	RS_LCD_PLOT = 'p',
	RS_LCD_CERCLE = 'c',
	RS_LCD_BITMAP = 'B',
	RS_LCD_BAR = 'b',				// barregraphe
	RS_LCD_BAR_TIME = 'y',		// barregraphe
	RS_BEEP = 'Y',
	RS_CODE_IR = 'i',
	RS_TEMP = 'f',					// temperature
	RS_PWM = 'F'					// PWM backlight
};


// FONT positif
enum { 
	font5x7p=0,
	font8x8p=1,
// FONT negatif
	font5x7n=0+2,
	font8x8n=1+2
};


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <math.h>
#include "serial.h"
#include "util_hex.h"
#include <qdatetime.h> 



class Lcd {
	public:
		Lcd(SerialCOM * portCom);
		~Lcd();

		// transmission commande
		bool send(char *data, unsigned char len);
	
		// Lecture temperature
		bool readTempRetry(signed char *temp, unsigned char adr, int nbRetry = 7);
		bool readTemp(signed char *temp, unsigned char adr) ;
	
		// Test si reception d'un code IR
		bool readIRretry(unsigned long *ircode, int nbRetry = 7);
		bool readIR(unsigned long *ircode);

		// Affiche le texte text
		bool TextRetry(char * text, unsigned char font, 
				unsigned char ligne, unsigned char col, int nbRetry = 7);
		bool Text(char * text, unsigned char font, 
				unsigned char ligne, unsigned char col);

		// Efface le LCD
		bool ClearRetry(int nbRetry = 7);
		bool Clear(void);

		// Affiche le bitmap
		bool BmpRetry(char * bitmap, unsigned char zone, int nbRetry);
		bool Bmp(char * bitmap, unsigned char zone);

		// Affiche le bitmap (122x32)
		bool Bmp122x32Retry(const unsigned char *image, int nbRetry = 7);
		int Bmp122x32(const unsigned char *image);

		// Affiche point
		bool PlotRetry(unsigned char x1, unsigned char y1, unsigned char fill, 
							int nbRetry);
		bool Plot(unsigned char x1, unsigned char y1, unsigned char fill);
		
		// Affiche ligne
		bool LigneRetry(unsigned char x1, unsigned char y1,
							unsigned char x2, unsigned char y2, unsigned char fill, int nbRetry = 7);
		bool Ligne(unsigned char x1, unsigned char y1,
							unsigned char x2, unsigned char y2, unsigned char fill);
		
		// Affiche rectangle
		bool RectRetry(unsigned char x1, unsigned char y1,
							unsigned char x2, unsigned char y2, int nbRetry=7);
		bool Rect(unsigned char x1, unsigned char y1,
							unsigned char x2, unsigned char y2);
		
		// Affiche rectanglePlein (0:efface, 1:rempli)
		bool RectFillRetry(unsigned char x1, unsigned char y1,
							unsigned char x2, unsigned char y2, unsigned char fill, int nbRetry=7);
		bool RectFill(unsigned char x1, unsigned char y1,
							unsigned char x2, unsigned char y2, unsigned char fill);

		// Affiche cercle
		bool CercleRetry(unsigned char cx, unsigned char cy,
						unsigned char radius, int nbRetry = 7);
		bool Cercle(unsigned char cx, unsigned char cy, unsigned char radius);

		// Affiche Bitmap eeprom
		bool BitmapEeRetry(unsigned char num,int nbRetry = 7);
		bool BitmapEe(unsigned char num);
		
		// Beep
		bool BeepRetry(unsigned char num,int nbRetry = 7);
		bool Beep();
	
		// PWM backlight
		bool PWMretry(unsigned char pwm,int nbRetry = 7);
		bool PWM(unsigned char pwm);
	
		// Affiche barregraph valeur, ligne, col
		bool BarRetry(unsigned char valeur, unsigned char ligne,
						  unsigned char col, int nbRetry = 7);
		bool Bar(unsigned char valeur, unsigned char ligne, unsigned char col);

		// Affiche barregraph temps valeur, ligne, col
		bool BarTimeRetry(unsigned char valeur, unsigned char ligne,
							unsigned char col, int nbRetry = 7);
		bool BarTime(unsigned char valeur, unsigned char ligne, unsigned char col);

		

	// Ecrit 32 octets max en hexa (ascii) 
		bool WriteEEretry(int adr, char *data, int nbData, int nbRetry = 7);
		bool WriteEE(int adr, char *data, int nbData);
		
		// Lit 32 octets en hexa (ascii) 
		bool ReadEEretry(int adr, char *data, int nbRetry = 7);
 		bool ReadEE(int adr, char *data);

	
	private:
		SerialCOM * _portCom;


};




#endif

