//========================================================================================
//   NOM     : maindialogimpl.cpp  		                                                                    
//   Date    : 04/11/2003                                                                     
//   Version : 1.00                                                                       
//   Auteur  : Sebastien JEFFROY                                                                          
//   Email   : sj29fr@yahoo.fr
//   Site    : http://sjeffroy.free.fr
//
//--------- GNU GPL LICENCE --------------------------------------------------
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
//----------------------------------------------------------------------------
//
//   Description : Boite de dialogue principale
//----------------------------------------------------------------------------
#include "maindialogimpl.h"

#include "lcd.h"

// uui.dll
/* 
 *  Constructs a maindialogImpl which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
maindialogImpl::maindialogImpl( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : maindialogBase( parent, name, modal, fl )
{

	// classe WinampCmd : permet de lire les infos de Winamp et de lui envoyer des commandes
	// creation d'un QThread de scrutation de Winamp toutes les 200ms
	_winCMD = new WinampCmd();
	if (!_winCMD->winampFound())					// Winamp trouve ?
	{
		QMessageBox::critical( 0, "WinIR",
                           QString("Winamp Introuvable") +
                           "\nrelancez Winamp puis WinIR\n"+
                           "\n\nFERMETURE DE L'APPLICATION" );
		exit(0);
	}
	Slider2->setValue(127);							// Volume
	///_winCMD->setSliderVolume(Slider2);			// ### BUG si modif sur ce pointeur !!



	// Port serie : Ouverture, Erreur si deja ouvert ou inexistant
	_portCom = new SerialCOM();
	int status = 0;
	///status = _portCom->InitComPort("COM1", 115200, 1, 1000);	// timeout = 1000ms		
	status = _portCom->InitComPort("COM1", 57600, 1, 100, 0);	// timeout = 100ms, pas de thread		
	if (status==COM_OPEN_ERROR) 
	{	// erreur d'ouverture port COM
		///QMessageBox::warning( this, tr("Erreur"), tr("Problme d'ouverture du port COM"));
		QMessageBox::critical( 0,	"WinIR",
 											"Problme d'ouverture du port COM\n"
											"Port COM dj ouvert\n"
											"ou inexistant !\n"
											"Modifier le port COM dans\n"
											"dans le fichier WINIR.INI");
		exit(0);
	}

	_thTitre = new CommandThread(this);
	_thTitre->setLblTitre(lblTitre, lblTime, TxtEeprom);	// ptr sur Lbl pour maj par thread
	_thTitre->setRefresh(400);						// 400ms (rafraichissement par thread
	_thTitre->setPortCom(_portCom);			// pointeur reference pour Thread CommandThread()
	_thTitre->start();								// demarrage du thread de scrutation de Winamp

	////lcd_init();	


}

/*  
 *  Destroys the object and frees any allocated resources
 */
maindialogImpl::~maindialogImpl()
{
    // no need to delete child widgets, Qt does it all for us
	_thTitre->stop(); // stoppe le thread 
}


// SLOTS
void maindialogImpl::play_click()
{
    _winCMD->cmdPlay();
	 _portCom->com_putstring("PLAY\n\r", 0);
	 ///lblTime->setText("Click PLAY");
}

void maindialogImpl::previous_click()
{
	_winCMD->cmdPrevious();
	///_portCom->com_putstring("PREVIOUS\n\r", 0);
}

void maindialogImpl::next_click()
{
	_winCMD->cmdNext();
	///_portCom->com_putstring("NEXT\n\r", 0);

/*	int stat;
	Lcd _lcd(_portCom);
	stat = _lcd.TestBmp();

	QString str;
	str.sprintf("Stat=%d", stat);
	TxtEeprom->setText(str);
*/
}

void maindialogImpl::stop_click()
{
	_winCMD->cmdStop();
/*	int stat;
	Lcd _lcd(_portCom);
	stat = _lcd.Clear();
	
	QString str;
	str.sprintf("Clear=%d", stat);
	TxtEeprom->setText(str);*/

/*	int stat;
	Lcd _lcd(_portCom);
	stat = _lcd.BarTime(50,3,0);
	stat += _lcd.Bar(50,3,30);


	QString  strName;
	_winCMD->getTitle(strName);
	stat += _lcd.Text((char*)(strName.latin1()),1,0,0);
//	stat += _lcd.Rect(0,0,31,121);

	QString str;
	str.sprintf("Stat=%d", stat);
	TxtEeprom->setText(str);*/


}

#include "lcd_images.cpp"
void maindialogImpl::updateInfo_click()
{
	/*QString  strName;
	_winCMD->getTitle(strName);
	lblTitre->setText(strName);

	int sample = _winCMD->getSampleRate();
	int bRate = _winCMD->getBitRate();	// 128 000 Bps
	int nbCh = _winCMD->getChannels();	

	lblTime->setText(tr("%1 KBPS  %1 KHz  %1 CH").arg(bRate).arg(sample).arg(nbCh));
	*/
	/*_thTitre->stop(); // stoppe le thread 
	TxtEeprom->setText(QString(""));

	char eeprom[1024*32], cmd[10];
	int i;
	for(i=0; i<1024*32; i+=64) {
		sprintf(cmd, "r%3X\n", i);
		_portCom->com_putstring(cmd, strlen((const char*)cmd));
		eeprom[i] = _portCom->com_getchar();
		TxtEeprom->append(QString(&eeprom[i]));
	}

	_thTitre->start(); // redemarre le thread */

	/////_winCMD->loadPlaylist("Xp.m3u");	// TEST 

	/*int stat;
	Lcd _lcd(_portCom);
	stat = _lcd.Text("Titi Test", 1, 3, 10);
	
	QString str;
	str.sprintf("Text=%d", stat);
	TxtEeprom->setText(str);*/


	Lcd _lcd(_portCom);
	_lcd.Bmp122x32Retry(image_ktv);

	HWND _hwnd;
	_hwnd = FindWindow("Media Player 2", NULL);
	if (_hwnd!=NULL) {
		SendMessage(_hwnd, WM_COMMAND, 68740,0);
	
	}
	/*
	// MediaPlayer
	Classe : "WMPlayerApp"
	98344 : CTRL+P PAUSE/Play
	98345 : CTRL+S STOP
	98353 : F8 Muet
	98352 : F9  VOL-
	98351 : F10 VOL+
	98318 : Alt+Entree FullScreen

	// MediaPlayer Ancien
	Classe : "Media Player 2"
	Nom : _Les_rois_mages.avi - Lecteur Windows Media
	68740 : ESPACE play/Pause
	66538 : Point : STOP
	68750 : Flech bas  Vol-
	68749 : Flech haut Vol+
	68751 : Ctrl+M mute
	68739 : Alt+Entree FullScreen

	// KTV! Xp
	Classe + Nom : "K!TV XP"
	65911 : Page UP Vol+
	65910 : Page Down Vol-
	65938 : M Mute
	65802 : + Ch+
	65803 : - Ch-
	65856 : Espace FullScreen


*/  
}

void maindialogImpl::volume_changed(int value)
{
	_winCMD->cmdVolume(value & 0xFF);
}
