//========================================================================================
//   NOM     : serial.cpp  		                                                                    
//   Date    : 04/11/2003                                                                     
//   Version : 1.00                                                                       
//   Auteur  : Sebastien JEFFROY                                                                          
//   Email   : sj29fr@yahoo.fr
//   Site    : http://sjeffroy.free.fr
//
//--------- GNU GPL LICENCE --------------------------------------------------
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
//----------------------------------------------------------------------------
//
//   Description : Routines de gestion du port serie
//----------------------------------------------------------------------------

#include "serial.h"

// Constructeur
SerialCOM::SerialCOM()
{
	_serial_in_head = 0;
	_serial_in_tail = 0;
}

// Destructeur
SerialCOM::~SerialCOM()
{
	stop();	// arrete le thread
	// ferme le port si ouvert
	if (_hCommPort != INVALID_HANDLE_VALUE) CloseComPort();
}


//----------------------------------------------------------------------------
// Thread de scrutation du port COM
//----------------------------------------------------------------------------
void SerialCOM::run()
{
	DWORD dwCount = 0;
	char c;

	_stopThread = 0;

	while(!_stopThread) 
	{
		if (dwCount==0) msleep(333);										// 200ms
		if (_hCommPort == INVALID_HANDLE_VALUE) dwCount = 0;
		else ReadFile((HANDLE)_hCommPort, &c, 1, &dwCount, NULL);// bloquant pendant (timeout * 1)
		if (dwCount == 1) {
			_serial_in_buffer[_serial_in_head++] = (unsigned char)c;
			if (_serial_in_head == SERIAL_IN_SIZE) _serial_in_head = 0;
		}
//		WriteFile(hConOut, &c, 1, &dwCount, NULL);					// echo
   }
}

//----------------------------------------------------------------------------
// Arret du Thread de scrutation du port COM
//----------------------------------------------------------------------------
void SerialCOM::stop() 
{
	_stopThread = 1;
}

//----------------------------------------------------------------------------
// Initialisation du port COM
//	status = InitComPort("COM1", 9600, 1, 100);	// timeout en ms		
//----------------------------------------------------------------------------
int 
SerialCOM::InitComPort(char *portstr, int baudrate, int dtr, int timeout) {
	COMMTIMEOUTS ctmoCommPort = {0};
	DCB dcbCommPort = {0};
 
	_hCommPort = CreateFile(portstr,						// ComPort
									GENERIC_READ | GENERIC_WRITE,
									0,								// not shared
									NULL,							// no security descriptor
									OPEN_EXISTING, 
									FILE_ATTRIBUTE_NORMAL,	///FILE_FLAG_OVERLAPPED, 
									NULL);
 	
	if (_hCommPort == INVALID_HANDLE_VALUE) return COM_OPEN_ERROR;
    
/*	//ctmoCommPort.ReadIntervalTimeout = MAXDWORD;	// ERREUR ctmo INVALID PARAM
	ctmoCommPort.ReadTotalTimeoutMultiplier = MAXDWORD;
	ctmoCommPort.ReadTotalTimeoutConstant = MAXDWORD;
	ctmoCommPort.WriteTotalTimeoutMultiplier = 0;
	ctmoCommPort.WriteTotalTimeoutConstant = 0;
	SetCommTimeouts(_hCommPort, &ctmoCommPort);
*/
	//Set the timeout conditions
	if (!GetCommTimeouts(_hCommPort, &ctmoCommPort)){
		return COM_OPEN_ERROR;//INVALID_HANDLE_VALUE;
	}
	else{
		ctmoCommPort.ReadIntervalTimeout = MAXDWORD;
		ctmoCommPort.ReadTotalTimeoutMultiplier = MAXDWORD;
		ctmoCommPort.ReadTotalTimeoutConstant = 4000;//ReadTimeOut;	// msec
	}

	if (!SetCommTimeouts(_hCommPort, &ctmoCommPort))
		return COM_OPEN_ERROR;//INVALID_HANDLE_VALUE;

	FillMemory(&dcbCommPort, sizeof(dcbCommPort), 0);
	GetCommState(_hCommPort, &dcbCommPort);			// Get the current state of the port
	dcbCommPort.BaudRate = baudrate;

	dcbCommPort.DCBlength = sizeof(DCB);				// ??
 	dcbCommPort.Parity = NOPARITY;
	dcbCommPort.StopBits = ONESTOPBIT;
	dcbCommPort.fDtrControl = DTR_CONTROL_DISABLE;
	dcbCommPort.fRtsControl = RTS_CONTROL_DISABLE;
	dcbCommPort.fOutX = FALSE;
	dcbCommPort.fInX = FALSE;
	dcbCommPort.fOutxCtsFlow = FALSE;
	dcbCommPort.fOutxDsrFlow = FALSE;
	dcbCommPort.ByteSize = 8;

	 
	 
	if (dtr) dcbCommPort.fDtrControl = DTR_CONTROL_ENABLE;
	else 		dcbCommPort.fDtrControl = DTR_CONTROL_DISABLE;		
	
	SetCommState(_hCommPort, &dcbCommPort);

	// cree un thread de scrutation du port COM
	start();														// demarre le thread QT
	return 1;
}



//----------------------------------------------------------------------------
// Retourne le nombre d'erreurs
//----------------------------------------------------------------------------
int 
SerialCOM::serial_get_error() {
	return _error_under;
}

//----------------------------------------------------------------------------
// Efface les erreurs
//----------------------------------------------------------------------------
void 
SerialCOM::serial_clear_error() {
	_error_under = 0;
}

//----------------------------------------------------------------------------
// Lit un octet dans la FILE, retource 0xFF si la FILE est vide
//----------------------------------------------------------------------------
unsigned char 
SerialCOM::com_getchar() {
	unsigned char c;
	if (_serial_in_head != _serial_in_tail) {
		c = _serial_in_buffer[_serial_in_tail++];
		if (_serial_in_tail == SERIAL_IN_SIZE) {
			_serial_in_tail = 0;
		}
		return c;
	} 
	else 
	{
		_error_under++;
		return 0xFF;	// com_getchar buffer underrun!
	}
}

//----------------------------------------------------------------------------
// Ecrit un caractere sur le port COM
//----------------------------------------------------------------------------
int 
SerialCOM::com_putchar(unsigned char ch) {
	char c;
	DWORD dwCount;
	
	if (_hCommPort == INVALID_HANDLE_VALUE) return 0;	// verifie si port com OK !
		
	c = ch;
	WriteFile(_hCommPort, &c, 1, &dwCount, NULL);
	usleep(100);//500);
	return dwCount;
}

//----------------------------------------------------------------------------
// Ecrit une string sur le port COM
// si nb=0 => strlen(str), sinon nb caracteres
// retourne le nombre de caracteres ecrits
//----------------------------------------------------------------------------
int 
SerialCOM::com_putstring(char *str, unsigned char nb) {
	unsigned char nbCar;
	DWORD BytesWritten =0, cptWord = 0;
	if (_hCommPort == INVALID_HANDLE_VALUE) return 0;	// verifie si port com OK !

	if (nb==0) nbCar = strlen(str);
	else nbCar = nb;

	/*do {
		WriteFile(_hCommPort, str + cptWord, 1, &BytesWritten, NULL);
		//usleep(500);//???
		cptWord += BytesWritten;
	}while(cptWord < nbCar); // OK trop rapide ??
	*/
	for(int i = 0; i < nb; i++) {
		com_putchar(str[i]);
	}
	
	return cptWord;
}

//----------------------------------------------------------------------------
// Retourne 0 si la File est vide (1 sinon)
//----------------------------------------------------------------------------
int 
SerialCOM::com_incoming() {
	if (_serial_in_head != _serial_in_tail) return 1;
	else return 0;
}

//----------------------------------------------------------------------------
// Vide la File
//----------------------------------------------------------------------------
void 
SerialCOM::com_in_clear() {
	_serial_in_head = _serial_in_tail;
}


//----------------------------------------------------------------------------
// Fermeture du port COM
//----------------------------------------------------------------------------
int 
SerialCOM::CloseComPort() {
	stop();												// stoppe le thread de scrutation de lecture	
	
	if (_hCommPort != INVALID_HANDLE_VALUE) 
		PurgeComm(_hCommPort, PURGE_RXABORT);	// purge le port		
	CloseHandle(_hCommPort);						// ferme le port
	return 1;
}


