//----------------------------------------------------------------------------
// Routines de gestion du port serie
// (Recupere dans DS1615src DALLAS(R) www.maxim-ic.com
//  Creation d'un thread de scrutation a l'initialisation du port COM
//----------------------------------------------------------------------------
#ifndef SERIAL_H
#define SERIAL_H

#include <windows.h>						// findwindow + HWND
#include <string.h>
#include <qthread.h>

#define COM_OPEN_ERROR -1


class SerialCOM : public QThread
{
public:
	SerialCOM(void);
	~SerialCOM();
	virtual void run();	// Thread	-> start() pour demarrer
	virtual void stop();	// Thread
	
	//----------------------------------------------------------------------------
	// Initialisation du port COM
	//	status = InitComPort("COM1", 9600, 1, 100);	// timeout en ms		
	// Si "COM5" remplacer par "//./COM5"	
	//----------------------------------------------------------------------------
	int InitComPort(char *portstr, int baudrate, int dtr, int timeout);

	//----------------------------------------------------------------------------
	// Fermeture du port COM
	//----------------------------------------------------------------------------
	int CloseComPort();

	//----------------------------------------------------------------------------
	// Lit un octet dans la FILE, retource 0xFF si la FILE est vide
	//----------------------------------------------------------------------------
	unsigned char com_getchar();

	//----------------------------------------------------------------------------
	// Ecrit un caractere sur le port COM
	//----------------------------------------------------------------------------
	int com_putchar(unsigned char ch);

	//----------------------------------------------------------------------------
	// Ecrit une string sur le port COM
	// si nb=0 => strlen(str), sinon nb caracteres
	// retourne le nombre de caracteres ecrits
	//----------------------------------------------------------------------------
	int com_putstring(char *str, unsigned char nb);

	//----------------------------------------------------------------------------
	// Retourne 0 si la File est vide (1 sinon)
	//----------------------------------------------------------------------------
	int com_incoming();

	//----------------------------------------------------------------------------
	// Vide la File
	//----------------------------------------------------------------------------
	void com_in_clear();

	//----------------------------------------------------------------------------
	// Retourne le nombre d'erreurs
	//----------------------------------------------------------------------------
	int serial_get_error(); 

	//----------------------------------------------------------------------------
	// Efface les erreurs
	//----------------------------------------------------------------------------
	void serial_clear_error();

private:
	DWORD WINAPI ReadComm(LPVOID hCommPort);
	// Buffer de stockage (FIFO)
	#define SERIAL_IN_SIZE 1024
	unsigned char _serial_in_buffer[SERIAL_IN_SIZE];
	unsigned int  _serial_in_head;
	unsigned int  _serial_in_tail;
	HANDLE _hCommPort;			// handle sur le port COM
	int _error_under;
	int _stopThread;
};


#endif

